@@ -1,4 +1,4 @@
-# $Id: ServiceType.pm,v 1.8 2007-12-13 17:10:02 mike Exp $
+# $Id: ServiceType.pm,v 1.9 2008-02-07 14:28:44 mike Exp $
 
 package Keystone::Resolver::DB::ServiceType;
 
@@ -15,12 +15,14 @@ sub table { "service_type" }
 sub fields { (id => undef,
 	      tag => undef,
 	      name => undef,
+	      plugin => undef,
 	      priority => undef,
 	      services => [ id => "Service", "service_type_id", "name" ],
 	      ) }
 
 sub search_fields { (tag => "t10",
 		     name => "t25",
+		     plugin => "t25",
 		     priority => "n5",
 		     ) }
 
@@ -34,6 +36,7 @@ sub display_fields { (id => "n",
 
 sub fulldisplay_fields { (tag => "c",
 			  name => "t",
+			  plugin => "t",
 			  priority => "n",
 			  services => "t",
 			  ) }
